package common

import (
	"fmt"
	"os"

	cnf "git.pbiernat.dev/egommerce/basket-service/pkg/config"
	srv "git.pbiernat.dev/egommerce/basket-service/pkg/server"
)

const (
	// defAppDomain     = "basket-svc"
	defAppName          = "basket-svc"
	defCacheAddr        = "api-cache:6379"
	defCachePassword    = "12345678"
	defDbURL            = "postgres://postgres:12345678@postgres-db:5432/egommerce"
	defEventBusURL      = "amqp://guest:guest@api-eventbus:56721"
	defKVNmspc          = "dev.egommerce/service/basket-svc"
	defLoggerAddr       = "api-logger:24224"
	defNetAddr          = ":80"
	defMongoDbURL       = "mongodb://mongodb:12345678@mongo-db:27017"
	defPathPrefix       = "/basket"
	defRegistryAddr     = "api-registry:8500"
	defEbEventsExchange = "api-events"
	defEbEventsQueue    = "basket-svc"
)

type Config struct {
	Base *srv.Config
	// AppID      string
	// AppName    string
	// NetAddr    string
	// PathPrefix string

	// IdleTimeout  time.Duration `json:"idle_timeout"`  // miliseconds
	// ReadTimeout  time.Duration `json:"read_timeout"`  // miliseconds
	// WriteTimeout time.Duration `json:"write_timeout"` // miliseconds

	DbURL            string `json:"db_url"`
	CacheAddr        string `json:"cache_addr"`
	CachePassword    string `json:"cache_password"`
	EventBusExchange string `json:"eventbus_exchange"`
	EventBusQueue    string `json:"eventbus_queue"`
	EventBusURL      string `json:"eventbus_url"`
	LoggerAddr       string `json:"logger_addr"`
	RegistryAddr     string

	// Fields with JSON mappings are available through Consul KV storage

	// 	Port             int
	// 	KVNamespace      string
	// 	MongoDbUrl       string `json:"mongodb_url"`
	// 	HttpReadTimeout  int    `json:"http_read_timeout"`
	// 	HttpWriteTimeout int    `json:"http_write_timeout"`
	// 	HttpIdleTimeout  int    `json:"http_idle_timeout"`
}

func NewConfig() *Config {
	c := new(Config)
	c.Base = new(srv.Config)

	c.Base.AppID, _ = os.Hostname()
	c.Base.AppName = cnf.GetEnv("APP_NAME", defAppName)
	c.Base.NetAddr = cnf.GetEnv("SERVER_ADDR", defNetAddr)
	c.Base.PathPrefix = cnf.GetEnv("APP_PATH_PREFIX", defPathPrefix)

	c.CacheAddr = cnf.GetEnv("CACHE_ADDR", defCacheAddr)
	c.CachePassword = cnf.GetEnv("CACHE_PASSWORD", defCachePassword)
	c.DbURL = cnf.GetEnv("DATABASE_URL", defDbURL)
	c.EventBusExchange = defEbEventsExchange
	c.EventBusURL = cnf.GetEnv("EVENTBUS_URL", defEventBusURL)
	c.LoggerAddr = cnf.GetEnv("LOGGER_ADDR", defLoggerAddr)
	c.RegistryAddr = cnf.GetEnv("REGISTRY_ADDR", defRegistryAddr)

	return c
}

func (c *Config) GetAppFullName() string {
	return fmt.Sprintf("%s_%s", c.Base.AppName, c.Base.AppID) // @TODO check if Base prop can be private
}

func (c *Config) GetListenAddr() string {
	return "" // @TODO: Implement me!
}
